@@ -32,6 +32,7 @@ static unsigned char ieee80211_oui[3] = { 0x00, 0x0f, 0xac };
 
 struct scan_params {
 	bool unknown;
+	enum print_ie_type type;
 };
 
 static int handle_scan(struct nl80211_state *state,
@@ -122,16 +123,8 @@ static void tab_on_first(bool *first)
 
 static void print_ssid(const uint8_t type, uint8_t len, const uint8_t *data)
 {
-	int i;
-
 	printf(" ");
-
-	for (i = 0; i < len; i++) {
-		if (isprint(data[i]))
-			printf("%c", data[i]);
-		else
-			printf("\\x%.2x", data[i]);
-	}
+	print_ssid_escaped(len, data);
 	printf("\n");
 }
 
@@ -486,6 +479,7 @@ struct ie_print {
 	const char *name;
 	void (*print)(const uint8_t type, uint8_t len, const uint8_t *data);
 	uint8_t minlen, maxlen;
+	uint8_t flags;
 };
 
 static void print_ie(const struct ie_print *p, const uint8_t type,
@@ -521,16 +515,16 @@ static void print_ie(const struct ie_print *p, const uint8_t type,
 }
 
 static const struct ie_print ieprinters[] = {
-	[0] = { "SSID", print_ssid, 0, 32, },
-	[1] = { "Supported rates", print_supprates, 0, 255, },
-	[3] = { "DS Paramater set", print_ds, 1, 1, },
+	[0] = { "SSID", print_ssid, 0, 32, BIT(PRINT_SCAN) | BIT(PRINT_LINK), },
+	[1] = { "Supported rates", print_supprates, 0, 255, BIT(PRINT_SCAN), },
+	[3] = { "DS Parameter set", print_ds, 1, 1, BIT(PRINT_SCAN), },
 	[5] = PRINT_IGN,
-	[7] = { "Country", print_country, 3, 255, },
-	[32] = { "Power constraint", print_powerconstraint, 1, 1, },
-	[42] = { "ERP", print_erp, 1, 255, },
-	[48] = { "RSN", print_rsn, 2, 255, },
-	[50] = { "Extended supported rates", print_supprates, 0, 255, },
-	[127] = { "Extended capabilities", print_capabilities, 0, 255, },
+	[7] = { "Country", print_country, 3, 255, BIT(PRINT_SCAN), },
+	[32] = { "Power constraint", print_powerconstraint, 1, 1, BIT(PRINT_SCAN), },
+	[42] = { "ERP", print_erp, 1, 255, BIT(PRINT_SCAN), },
+	[48] = { "RSN", print_rsn, 2, 255, BIT(PRINT_SCAN), },
+	[50] = { "Extended supported rates", print_supprates, 0, 255, BIT(PRINT_SCAN), },
+	[127] = { "Extended capabilities", print_capabilities, 0, 255, BIT(PRINT_SCAN), },
 };
 
 static void print_wifi_wpa(const uint8_t type, uint8_t len, const uint8_t *data)
@@ -538,6 +532,48 @@ static void print_wifi_wpa(const uint8_t type, uint8_t len, const uint8_t *data)
 	print_rsn_ie("TKIP", "IEEE 802.1X", len, data);
 }
 
+static bool print_wifi_wmm_param(const uint8_t *data, uint8_t len)
+{
+	int i;
+	static const char *aci_tbl[] = { "BE", "BK", "VI", "VO" };
+
+	if (len < 19)
+		goto invalid;
+
+	if (data[0] != 1) {
+		printf("Parameter: not version 1: ");
+		return false;
+	}
+
+	printf("\t* Parameter version 1");
+
+	data++;
+
+	if (data[0] & 0x80)
+		printf("\n\t\t* u-APSD");
+
+	data += 2;
+
+	for (i = 0; i < 4; i++) {
+		printf("\n\t\t* %s:", aci_tbl[(data[0] >> 5) & 3]);
+		if (data[4] & 0x10)
+			printf(" acm");
+		printf(" CW %d-%d", (1 << (data[1] & 0xf)) - 1,
+				    (1 << (data[1] >> 4)) - 1);
+		printf(", AIFSN %d", data[0] & 0xf);
+		if (data[2] | data[3])
+			printf(", TXOP %d usec", (data[2] + (data[3] << 8)) * 32);
+		data += 4;
+	}
+
+	printf("\n");
+	return true;
+
+ invalid:
+ 	printf("invalid: ");
+ 	return false;
+}
+
 static void print_wifi_wmm(const uint8_t type, uint8_t len, const uint8_t *data)
 {
 	int i;
@@ -547,15 +583,16 @@ static void print_wifi_wmm(const uint8_t type, uint8_t len, const uint8_t *data)
 		printf(" information:");
 		break;
 	case 0x01:
-		printf(" parameter:");
+		if (print_wifi_wmm_param(data + 1, len - 1))
+			return;
 		break;
 	default:
 		printf(" type %d:", data[0]);
 		break;
 	}
 
-	for(i = 0; i < len - 1; i++)
-		printf(" %.02x", data[i + 1]);
+	for(i = 1; i < len; i++)
+		printf(" %.02x", data[i]);
 	printf("\n");
 }
 
@@ -588,9 +625,9 @@ static void print_wifi_wps(const uint8_t type, uint8_t len, const uint8_t *data)
 			printf("\t * Model: %.*s\n", sublen, data + 4);
 			break;
 		case 0x1057: {
-			__u16 val = (data[4] << 8) | data[5];
+			__u8 val = data[4];
 			tab_on_first(&first);
-			printf("\t * AP setup locked: 0x%.4x\n", val);
+			printf("\t * AP setup locked: 0x%.2x\n", val);
 			break;
 		}
 		case 0x1008: {
@@ -638,13 +675,13 @@ static void print_wifi_wps(const uint8_t type, uint8_t len, const uint8_t *data)
 }
 
 static const struct ie_print wifiprinters[] = {
-	[1] = { "WPA", print_wifi_wpa, 2, 255, },
-	[2] = { "WMM", print_wifi_wmm, 1, 255, },
-	[4] = { "WPS", print_wifi_wps, 0, 255, },
+	[1] = { "WPA", print_wifi_wpa, 2, 255, BIT(PRINT_SCAN), },
+	[2] = { "WMM", print_wifi_wmm, 1, 255, BIT(PRINT_SCAN), },
+	[4] = { "WPS", print_wifi_wps, 0, 255, BIT(PRINT_SCAN), },
 };
 
 static void print_vendor(unsigned char len, unsigned char *data,
-			 struct scan_params *params)
+			 bool unknown, enum print_ie_type ptype)
 {
 	int i;
 
@@ -657,11 +694,13 @@ static void print_vendor(unsigned char len, unsigned char *data,
 	}
 
 	if (len >= 4 && memcmp(data, wifi_oui, 3) == 0) {
-		if (data[3] < ARRAY_SIZE(wifiprinters) && wifiprinters[data[3]].name) {
+		if (data[3] < ARRAY_SIZE(wifiprinters) &&
+		    wifiprinters[data[3]].name &&
+		    wifiprinters[data[3]].flags & BIT(ptype)) {
 			print_ie(&wifiprinters[data[3]], data[3], len - 4, data + 4);
 			return;
 		}
-		if (!params->unknown)
+		if (!unknown)
 			return;
 		printf("\tWiFi OUI %#.2x, data:", data[3]);
 		for(i = 0; i < len - 4; i++)
@@ -670,7 +709,7 @@ static void print_vendor(unsigned char len, unsigned char *data,
 		return;
 	}
 
-	if (!params->unknown)
+	if (!unknown)
 		return;
 
 	printf("\tVendor specific: OUI %.2x:%.2x:%.2x, data:",
@@ -680,14 +719,17 @@ static void print_vendor(unsigned char len, unsigned char *data,
 	printf("\n");
 }
 
-static void print_ies(unsigned char *ie, int ielen, struct scan_params *params)
+void print_ies(unsigned char *ie, int ielen, bool unknown,
+	       enum print_ie_type ptype)
 {
 	while (ielen >= 2 && ielen >= ie[1]) {
-		if (ie[0] < ARRAY_SIZE(ieprinters) && ieprinters[ie[0]].name) {
+		if (ie[0] < ARRAY_SIZE(ieprinters) &&
+		    ieprinters[ie[0]].name &&
+		    ieprinters[ie[0]].flags & BIT(ptype)) {
 			print_ie(&ieprinters[ie[0]], ie[0], ie[1], ie + 2);
 		} else if (ie[0] == 221 /* vendor */) {
-			print_vendor(ie[1], ie + 2, params);
-		} else if (params->unknown) {
+			print_vendor(ie[1], ie + 2, unknown, ptype);
+		} else if (unknown) {
 			int i;
 
 			printf("\tUnknown IE (%d):", ie[0]);
@@ -715,7 +757,9 @@ static int print_bss_handler(struct nl_msg *msg, void *arg)
 		[NL80211_BSS_INFORMATION_ELEMENTS] = { },
 		[NL80211_BSS_SIGNAL_MBM] = { .type = NLA_U32 },
 		[NL80211_BSS_SIGNAL_UNSPEC] = { .type = NLA_U8 },
+		[NL80211_BSS_STATUS] = { .type = NLA_U32 },
 	};
+	struct scan_params *params = arg;
 
 	nla_parse(tb, NL80211_ATTR_MAX, genlmsg_attrdata(gnlh, 0),
 		  genlmsg_attrlen(gnlh, 0), NULL);
@@ -736,7 +780,26 @@ static int print_bss_handler(struct nl_msg *msg, void *arg)
 
 	mac_addr_n2a(mac_addr, nla_data(bss[NL80211_BSS_BSSID]));
 	if_indextoname(nla_get_u32(tb[NL80211_ATTR_IFINDEX]), dev);
-	printf("BSS %s (on %s)\n", mac_addr, dev);
+	printf("BSS %s (on %s)", mac_addr, dev);
+
+	if (bss[NL80211_BSS_STATUS]) {
+		switch (nla_get_u32(bss[NL80211_BSS_STATUS])) {
+		case NL80211_BSS_STATUS_AUTHENTICATED:
+			printf(" -- authenticated");
+			break;
+		case NL80211_BSS_STATUS_ASSOCIATED:
+			printf(" -- associated");
+			break;
+		case NL80211_BSS_STATUS_IBSS_JOINED:
+			printf(" -- joined");
+			break;
+		default:
+			printf(" -- unknown status: %d",
+				nla_get_u32(bss[NL80211_BSS_STATUS]));
+			break;
+		}
+	}
+	printf("\n");
 
 	if (bss[NL80211_BSS_TSF]) {
 		unsigned long long tsf;
@@ -789,7 +852,7 @@ static int print_bss_handler(struct nl_msg *msg, void *arg)
 	if (bss[NL80211_BSS_INFORMATION_ELEMENTS])
 		print_ies(nla_data(bss[NL80211_BSS_INFORMATION_ELEMENTS]),
 			  nla_len(bss[NL80211_BSS_INFORMATION_ELEMENTS]),
-			  arg);
+			  params->unknown, params->type);
 
 	return NL_SKIP;
 }
@@ -808,6 +871,8 @@ static int handle_scan_dump(struct nl80211_state *state,
 	if (argc == 1 && !strcmp(argv[0], "-u"))
 		scan_params.unknown = true;
 
+	scan_params.type = PRINT_SCAN;
+
 	nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, print_bss_handler,
 		  &scan_params);
 	return 0;
